package com.example.refactor.adapter;

import com.example.refactor.entity.User;
import com.example.refactor.entity.v2.EmployeeInfo;
import com.example.refactor.entity.v2.UserProfile;
import com.example.refactor.entity.v2.UserSecurity;
import org.springframework.stereotype.Component;

/**
 * 用户适配器实现类
 * 职责：实现新旧User实体之间的具体转换逻辑
 */
@Component
public class UserAdapterImpl implements UserAdapter {
    
    @Override
    public UserProfile toUserProfile(User user) {
        if (user == null) return null;
        
        UserProfile profile = new UserProfile();
        profile.setId(user.getId());
        profile.setName(user.getName());
        profile.setEmail(user.getEmail());
        profile.setPhone(user.getPhone());
        profile.setAddress(user.getAddress());
        profile.setEmergencyContact(user.getEmergencyContact());
        profile.setCreateTime(user.getCreateTime());
        profile.setUpdateTime(user.getUpdateTime());
        
        return profile;
    }
    
    @Override
    public EmployeeInfo toEmployeeInfo(User user, UserProfile userProfile) {
        if (user == null || userProfile == null) return null;
        
        // 只有当用户有工作相关信息时才创建EmployeeInfo
        if (user.getDepartment() == null && user.getPosition() == null && user.getSalary() == null) {
            return null;
        }
        
        EmployeeInfo employeeInfo = new EmployeeInfo();
        employeeInfo.setUserProfile(userProfile);
        employeeInfo.setDepartment(user.getDepartment());
        employeeInfo.setPosition(user.getPosition());
        employeeInfo.setSalary(user.getSalary());
        employeeInfo.setHireDate(user.getCreateTime()); // 假设创建时间为入职时间
        employeeInfo.setUpdateTime(user.getUpdateTime());
        
        return employeeInfo;
    }
    
    @Override
    public UserSecurity toUserSecurity(User user, UserProfile userProfile) {
        if (user == null || userProfile == null) return null;
        
        UserSecurity security = new UserSecurity();
        security.setUserProfile(userProfile);
        
        // 转换状态
        if ("ACTIVE".equals(user.getStatus())) {
            security.setStatus(UserSecurity.UserStatus.ACTIVE);
        } else if ("LOCKED".equals(user.getStatus())) {
            security.setStatus(UserSecurity.UserStatus.LOCKED);
        } else if ("DELETED".equals(user.getStatus())) {
            security.setStatus(UserSecurity.UserStatus.DELETED);
        } else {
            security.setStatus(UserSecurity.UserStatus.SUSPENDED);
        }
        
        security.setLastLoginIp(user.getLastLoginIp());
        security.setLastLoginTime(user.getLastLoginTime());
        security.setUpdateTime(user.getUpdateTime());
        
        return security;
    }
    
    @Override
    public User toLegacyUser(UserProfile userProfile, EmployeeInfo employeeInfo, UserSecurity userSecurity) {
        if (userProfile == null) return null;
        
        User user = new User();
        
        // 基础信息
        user.setId(userProfile.getId());
        user.setName(userProfile.getName());
        user.setEmail(userProfile.getEmail());
        user.setPhone(userProfile.getPhone());
        user.setAddress(userProfile.getAddress());
        user.setEmergencyContact(userProfile.getEmergencyContact());
        user.setCreateTime(userProfile.getCreateTime());
        user.setUpdateTime(userProfile.getUpdateTime());
        
        // 员工信息
        if (employeeInfo != null) {
            user.setDepartment(employeeInfo.getDepartment());
            user.setPosition(employeeInfo.getPosition());
            user.setSalary(employeeInfo.getSalary());
        }
        
        // 安全信息
        if (userSecurity != null) {
            user.setStatus(userSecurity.getStatus().name());
            user.setLastLoginIp(userSecurity.getLastLoginIp());
            user.setLastLoginTime(userSecurity.getLastLoginTime());
        }
        
        return user;
    }
    
    @Override
    public User toLegacyUserBasic(UserProfile userProfile, UserSecurity userSecurity) {
        return toLegacyUser(userProfile, null, userSecurity);
    }
}