package com.example.refactor.adapter;

import com.example.refactor.entity.User;
import com.example.refactor.entity.UserV2;
import org.springframework.stereotype.Component;

/**
 * User版本间的适配器
 * 演示如何处理实体版本升级时的数据转换
 * 关键功能：
 * 1. 旧版本User到新版本UserV2的转换
 * 2. 新版本UserV2到旧版本User的转换
 * 3. 字段映射和类型转换
 * 4. 向后兼容性保证
 */
@Component
public class UserVersionAdapter {
    
    /**
     * 将旧版本User转换为新版本UserV2
     * 处理字段变更：
     * - name -> firstName + middleName + lastName
     * - salary (Double) -> salaryAmount (BigDecimal)
     * - address (String) -> address (Address对象)
     */
    public UserV2 convertToV2(User userV1) {
        if (userV1 == null) {
            return null;
        }
        
        UserV2 userV2 = new UserV2();
        
        // 基本字段映射
        userV2.setId(userV1.getId());
        userV2.setEmail(userV1.getEmail());
        userV2.setPhone(userV1.getPhone());
        
        // 姓名字段拆分处理
        if (userV1.getName() != null) {
            userV2.setName(userV1.getName());  // 使用兼容性方法
        }
        
        // 地址转换
        if (userV1.getAddress() != null) {
            userV2.setAddress(userV1.getAddress());  // 使用兼容性方法
        }
        
        // 员工信息
        userV2.setDepartment(userV1.getDepartment());
        userV2.setPosition(userV1.getPosition());
        
        // 薪资类型转换：Double -> BigDecimal
        if (userV1.getSalary() != null) {
            userV2.setSalary(userV1.getSalary());  // 使用兼容性方法
        }
        
        userV2.setHireDate(userV1.getHireDate());
        
        // 安全信息
        userV2.setStatus(userV1.getStatus());
        userV2.setLastLoginTime(userV1.getLastLoginTime());
        userV2.setLastLoginIp(userV1.getLastLoginIp());
        userV2.setFailedLoginAttempts(userV1.getFailedLoginAttempts());
        userV2.setLockedUntil(userV1.getLockedUntil());
        userV2.setIsLocked(userV1.getIsLocked());
        
        // 审计字段
        userV2.setCreatedAt(userV1.getCreatedAt());
        userV2.setUpdatedAt(userV1.getUpdatedAt());
        
        // 废弃字段处理
        if (userV1.getFax() != null) {
            userV2.setFax(userV1.getFax());
        }
        
        return userV2;
    }
    
    /**
     * 将新版本UserV2转换为旧版本User
     * 确保向后兼容性
     */
    public User convertToV1(UserV2 userV2) {
        if (userV2 == null) {
            return null;
        }
        
        User userV1 = new User();
        
        // 基本字段映射
        userV1.setId(userV2.getId());
        userV1.setEmail(userV2.getEmail());
        userV1.setPhone(userV2.getPhone());
        
        // 姓名字段合并处理
        userV1.setName(userV2.getName());  // 使用兼容性方法
        
        // 地址转换
        userV1.setAddress(userV2.getAddress());  // 使用兼容性方法
        
        // 员工信息
        userV1.setDepartment(userV2.getDepartment());
        userV1.setPosition(userV2.getPosition());
        
        // 薪资类型转换：BigDecimal -> Double
        userV1.setSalary(userV2.getSalary());  // 使用兼容性方法
        
        userV1.setHireDate(userV2.getHireDate());
        
        // 安全信息
        userV1.setStatus(userV2.getStatus());
        userV1.setLastLoginTime(userV2.getLastLoginTime());
        userV1.setLastLoginIp(userV2.getLastLoginIp());
        userV1.setFailedLoginAttempts(userV2.getFailedLoginAttempts());
        userV1.setLockedUntil(userV2.getLockedUntil());
        userV1.setIsLocked(userV2.getIsLocked());
        
        // 审计字段
        userV1.setCreatedAt(userV2.getCreatedAt());
        userV1.setUpdatedAt(userV2.getUpdatedAt());
        
        // 废弃字段处理
        userV1.setFax(userV2.getFax());
        
        return userV1;
    }
    
    /**
     * 批量转换：V1 -> V2
     */
    public java.util.List<UserV2> convertListToV2(java.util.List<User> usersV1) {
        if (usersV1 == null) {
            return null;
        }
        
        return usersV1.stream()
                .map(this::convertToV2)
                .collect(java.util.stream.Collectors.toList());
    }
    
    /**
     * 批量转换：V2 -> V1
     */
    public java.util.List<User> convertListToV1(java.util.List<UserV2> usersV2) {
        if (usersV2 == null) {
            return null;
        }
        
        return usersV2.stream()
                .map(this::convertToV1)
                .collect(java.util.stream.Collectors.toList());
    }
    
    /**
     * 部分更新：将UserV2的变更应用到UserV1
     * 用于渐进式迁移场景
     */
    public void applyV2ChangesToV1(User userV1, UserV2 userV2) {
        if (userV1 == null || userV2 == null) {
            return;
        }
        
        // 只更新非空字段
        if (userV2.getEmail() != null) {
            userV1.setEmail(userV2.getEmail());
        }
        
        if (userV2.getPhone() != null) {
            userV1.setPhone(userV2.getPhone());
        }
        
        // 姓名处理
        String fullName = userV2.getName();
        if (fullName != null && !fullName.trim().isEmpty()) {
            userV1.setName(fullName);
        }
        
        // 地址处理
        String address = userV2.getAddress();
        if (address != null) {
            userV1.setAddress(address);
        }
        
        // 员工信息
        if (userV2.getDepartment() != null) {
            userV1.setDepartment(userV2.getDepartment());
        }
        
        if (userV2.getPosition() != null) {
            userV1.setPosition(userV2.getPosition());
        }
        
        // 薪资处理
        Double salary = userV2.getSalary();
        if (salary != null) {
            userV1.setSalary(salary);
        }
        
        if (userV2.getHireDate() != null) {
            userV1.setHireDate(userV2.getHireDate());
        }
        
        // 安全信息
        if (userV2.getStatus() != null) {
            userV1.setStatus(userV2.getStatus());
        }
        
        if (userV2.getLastLoginTime() != null) {
            userV1.setLastLoginTime(userV2.getLastLoginTime());
        }
        
        if (userV2.getLastLoginIp() != null) {
            userV1.setLastLoginIp(userV2.getLastLoginIp());
        }
        
        if (userV2.getFailedLoginAttempts() != null) {
            userV1.setFailedLoginAttempts(userV2.getFailedLoginAttempts());
        }
        
        if (userV2.getLockedUntil() != null) {
            userV1.setLockedUntil(userV2.getLockedUntil());
        }
        
        if (userV2.getIsLocked() != null) {
            userV1.setIsLocked(userV2.getIsLocked());
        }
        
        // 更新时间戳
        userV1.setUpdatedAt(java.time.LocalDateTime.now());
    }
    
    /**
     * 验证转换的完整性
     * 用于测试和调试
     */
    public boolean validateConversion(User original, UserV2 converted) {
        if (original == null && converted == null) {
            return true;
        }
        
        if (original == null || converted == null) {
            return false;
        }
        
        // 验证关键字段
        return java.util.Objects.equals(original.getId(), converted.getId()) &&
               java.util.Objects.equals(original.getEmail(), converted.getEmail()) &&
               java.util.Objects.equals(original.getName(), converted.getName()) &&
               java.util.Objects.equals(original.getSalary(), converted.getSalary());
    }
    
    /**
     * 获取转换统计信息
     * 用于监控和调试
     */
    public ConversionStats getConversionStats(java.util.List<User> usersV1) {
        if (usersV1 == null) {
            return new ConversionStats(0, 0, 0);
        }
        
        int total = usersV1.size();
        int successful = 0;
        int failed = 0;
        
        for (User user : usersV1) {
            try {
                UserV2 converted = convertToV2(user);
                if (validateConversion(user, converted)) {
                    successful++;
                } else {
                    failed++;
                }
            } catch (Exception e) {
                failed++;
            }
        }
        
        return new ConversionStats(total, successful, failed);
    }
    
    /**
     * 转换统计信息类
     */
    public static class ConversionStats {
        private final int total;
        private final int successful;
        private final int failed;
        
        public ConversionStats(int total, int successful, int failed) {
            this.total = total;
            this.successful = successful;
            this.failed = failed;
        }
        
        public int getTotal() { return total; }
        public int getSuccessful() { return successful; }
        public int getFailed() { return failed; }
        public double getSuccessRate() { 
            return total > 0 ? (double) successful / total * 100 : 0;
        }
        
        @Override
        public String toString() {
            return String.format("ConversionStats{total=%d, successful=%d, failed=%d, successRate=%.2f%%}",
                    total, successful, failed, getSuccessRate());
        }
    }
}