package com.example.refactor.controller;

import com.example.refactor.dto.UserDTO;
import com.example.refactor.dto.UserMapper;
import com.example.refactor.entity.User;
import com.example.refactor.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 用户控制器重构演示
 * 展示如何将直接返回User实体的方法改造为DTO模式
 * 
 * 重构步骤：
 * 1. 保留原始方法（标记为@Deprecated）
 * 2. 创建新的DTO版本方法
 * 3. 逐步迁移客户端到新API
 * 4. 最终删除废弃方法
 */
@RestController
@RequestMapping("/api/refactor-demo/users")
public class UserControllerRefactorDemo {
    
    @Autowired
    private UserManagementService userManagementService;
    
    @Autowired
    private HRService hrService;
    
    @Autowired
    private UserMapper userMapper;
    
    // ==================== 原始方法（问题版本） ====================
    
    /**
     * 原始方法：直接返回User实体
     * 问题：User实体变更会直接影响API响应
     */
    @GetMapping("/legacy/{id}")
    @Deprecated
    public ResponseEntity<User> getUserLegacy(@PathVariable Long id) {
        User user = userManagementService.getUserById(id);
        if (user != null) {
            // ❌ 问题：直接暴露实体，无法控制字段暴露
            return ResponseEntity.ok(user);
        }
        return ResponseEntity.notFound().build();
    }
    
    /**
     * 原始方法：获取所有用户
     */
    @GetMapping("/legacy")
    @Deprecated
    public ResponseEntity<List<User>> getAllUsersLegacy() {
        List<User> users = userManagementService.getAllActiveUsers();
        // ❌ 问题：直接返回实体列表
        return ResponseEntity.ok(users);
    }
    
    /**
     * 原始方法：创建用户
     */
    @PostMapping("/legacy")
    @Deprecated
    public ResponseEntity<User> createUserLegacy(@RequestBody CreateUserRequest request) {
        User user = userManagementService.createUser(
            request.getName(),
            request.getEmail(),
            request.getPhone()
        );
        // ❌ 问题：直接返回新创建的实体
        return ResponseEntity.ok(user);
    }
    
    /**
     * 原始方法：更新用户
     */
    @PutMapping("/legacy/{id}")
    @Deprecated
    public ResponseEntity<User> updateUserLegacy(@PathVariable Long id, @RequestBody UpdateUserRequest request) {
        User user = userManagementService.updateUser(
            id,
            request.getName(),
            request.getEmail(),
            request.getPhone()
        );
        if (user != null) {
            // ❌ 问题：直接返回更新后的实体
            return ResponseEntity.ok(user);
        }
        return ResponseEntity.notFound().build();
    }
    
    // ==================== 重构后的方法（推荐版本） ====================
    
    /**
     * 重构后：使用DTO返回用户信息
     * 优势：User实体变更不会直接影响API响应
     */
    @GetMapping("/{id}")
    public ResponseEntity<UserDTO> getUser(@PathVariable Long id) {
        User user = userManagementService.getUserById(id);
        if (user != null) {
            // ✅ 通过Mapper转换为DTO，隔离实体变更影响
            UserDTO userDTO = userMapper.toDTO(user);
            return ResponseEntity.ok(userDTO);
        }
        return ResponseEntity.notFound().build();
    }
    
    /**
     * 重构后：获取所有用户（返回DTO列表）
     */
    @GetMapping
    public ResponseEntity<List<UserDTO>> getAllUsers() {
        List<User> users = userManagementService.getAllActiveUsers();
        // ✅ 批量转换为DTO
        List<UserDTO> userDTOs = users.stream()
                .map(userMapper::toDTO)
                .collect(Collectors.toList());
        return ResponseEntity.ok(userDTOs);
    }
    
    /**
     * 重构后：创建用户（返回DTO）
     */
    @PostMapping
    public ResponseEntity<UserDTO> createUser(@RequestBody CreateUserRequest request) {
        User user = userManagementService.createUser(
            request.getName(),
            request.getEmail(),
            request.getPhone()
        );
        // ✅ 转换为DTO返回
        UserDTO userDTO = userMapper.toDTO(user);
        return ResponseEntity.ok(userDTO);
    }
    
    /**
     * 重构后：更新用户（返回DTO）
     */
    @PutMapping("/{id}")
    public ResponseEntity<UserDTO> updateUser(@PathVariable Long id, @RequestBody UpdateUserRequest request) {
        User user = userManagementService.updateUser(
            id,
            request.getName(),
            request.getEmail(),
            request.getPhone()
        );
        if (user != null) {
            // ✅ 转换为DTO返回
            UserDTO userDTO = userMapper.toDTO(user);
            return ResponseEntity.ok(userDTO);
        }
        return ResponseEntity.notFound().build();
    }
    
    // ==================== 演示User实体变更的影响 ====================
    
    /**
     * 演示：当User实体添加新字段时的处理
     * 假设User添加了middleName字段
     */
    @GetMapping("/{id}/with-middle-name")
    public ResponseEntity<UserDTO> getUserWithMiddleName(@PathVariable Long id) {
        User user = userManagementService.getUserById(id);
        if (user != null) {
            UserDTO userDTO = userMapper.toDTO(user);
            
            // ✅ 在Mapper中可以控制如何处理新字段
            // 例如：将firstName + middleName + lastName组合为fullName
            // 或者选择性地暴露middleName字段
            
            return ResponseEntity.ok(userDTO);
        }
        return ResponseEntity.notFound().build();
    }
    
    /**
     * 演示：处理敏感字段的过滤
     * 返回公开信息，过滤敏感数据
     */
    @GetMapping("/{id}/public")
    public ResponseEntity<UserPublicDTO> getUserPublicInfo(@PathVariable Long id) {
        User user = userManagementService.getUserById(id);
        if (user != null) {
            // ✅ 使用专门的公开信息DTO，只暴露非敏感字段
            UserPublicDTO publicDTO = createPublicDTO(user);
            return ResponseEntity.ok(publicDTO);
        }
        return ResponseEntity.notFound().build();
    }
    
    /**
     * 演示：版本化的API响应
     * 支持不同版本的客户端
     */
    @GetMapping("/{id}/v2")
    public ResponseEntity<UserDTOV2> getUserV2(@PathVariable Long id) {
        User user = userManagementService.getUserById(id);
        if (user != null) {
            // ✅ 使用V2版本的DTO，支持新的字段结构
            UserDTOV2 userDTOV2 = createUserDTOV2(user);
            return ResponseEntity.ok(userDTOV2);
        }
        return ResponseEntity.notFound().build();
    }
    
    // ==================== 辅助方法和DTO类 ====================
    
    private UserPublicDTO createPublicDTO(User user) {
        UserPublicDTO dto = new UserPublicDTO();
        dto.setId(user.getId());
        dto.setName(user.getName());
        dto.setEmail(maskEmail(user.getEmail()));  // 脱敏处理
        // 不暴露敏感信息如薪资、登录信息等
        return dto;
    }
    
    private UserDTOV2 createUserDTOV2(User user) {
        UserDTOV2 dto = new UserDTOV2();
        dto.setId(user.getId());
        
        // V2版本支持分离的姓名字段
        String[] nameParts = user.getName().split(" ");
        dto.setFirstName(nameParts.length > 0 ? nameParts[0] : "");
        dto.setLastName(nameParts.length > 1 ? nameParts[nameParts.length - 1] : "");
        
        dto.setEmail(user.getEmail());
        dto.setPhone(user.getPhone());
        
        return dto;
    }
    
    private String maskEmail(String email) {
        if (email == null || !email.contains("@")) {
            return email;
        }
        String[] parts = email.split("@");
        String username = parts[0];
        String domain = parts[1];
        
        if (username.length() <= 2) {
            return email;
        }
        
        return username.charAt(0) + "***" + username.charAt(username.length() - 1) + "@" + domain;
    }
    
    // ==================== DTO类定义 ====================
    
    /**
     * 公开信息DTO - 只包含非敏感字段
     */
    public static class UserPublicDTO {
        private Long id;
        private String name;
        private String email;  // 脱敏后的邮箱
        
        // Getters and Setters
        public Long getId() { return id; }
        public void setId(Long id) { this.id = id; }
        public String getName() { return name; }
        public void setName(String name) { this.name = name; }
        public String getEmail() { return email; }
        public void setEmail(String email) { this.email = email; }
    }
    
    /**
     * V2版本DTO - 支持新的字段结构
     */
    public static class UserDTOV2 {
        private Long id;
        private String firstName;
        private String lastName;
        private String email;
        private String phone;
        
        // Getters and Setters
        public Long getId() { return id; }
        public void setId(Long id) { this.id = id; }
        public String getFirstName() { return firstName; }
        public void setFirstName(String firstName) { this.firstName = firstName; }
        public String getLastName() { return lastName; }
        public void setLastName(String lastName) { this.lastName = lastName; }
        public String getEmail() { return email; }
        public void setEmail(String email) { this.email = email; }
        public String getPhone() { return phone; }
        public void setPhone(String phone) { this.phone = phone; }
    }
    
    /**
     * 请求DTO类
     */
    public static class CreateUserRequest {
        private String name;
        private String email;
        private String phone;
        
        // Getters and Setters
        public String getName() { return name; }
        public void setName(String name) { this.name = name; }
        public String getEmail() { return email; }
        public void setEmail(String email) { this.email = email; }
        public String getPhone() { return phone; }
        public void setPhone(String phone) { this.phone = phone; }
    }
    
    public static class UpdateUserRequest {
        private String name;
        private String email;
        private String phone;
        
        // Getters and Setters
        public String getName() { return name; }
        public void setName(String name) { this.name = name; }
        public String getEmail() { return email; }
        public void setEmail(String email) { this.email = email; }
        public String getPhone() { return phone; }
        public void setPhone(String phone) { this.phone = phone; }
    }
}