package com.example.refactor.controller;

import com.example.refactor.dto.UserDTO;
import com.example.refactor.dto.UserMapper;
import com.example.refactor.entity.User;
import com.example.refactor.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 用户控制器V2 - 演示如何减少对User实体的直接依赖
 * 关键改进：
 * 1. 使用DTO替代直接暴露实体
 * 2. 通过Mapper进行实体与DTO转换
 * 3. 控制器层不直接依赖具体的User实体结构
 */
@RestController
@RequestMapping("/api/v2/users")
public class UserControllerV2 {
    
    @Autowired
    private UserManagementService userManagementService;
    
    @Autowired
    private HRService hrService;
    
    @Autowired
    private SecurityService securityService;
    
    @Autowired
    private NotificationService notificationService;
    
    @Autowired
    private UserMapper userMapper;
    
    @Value("${app.user.architecture:v1}")
    private String currentArchitecture;
    
    @GetMapping("/architecture")
    public ResponseEntity<Map<String, String>> getArchitectureInfo() {
        Map<String, String> info = new HashMap<>();
        info.put("current_architecture", currentArchitecture);
        info.put("description", currentArchitecture.equals("v2") ? 
                "使用新的分离架构" : "使用原始单体架构");
        info.put("controller_version", "V2 - 使用DTO模式，减少实体依赖");
        return ResponseEntity.ok(info);
    }
    
    /**
     * 创建用户 - 返回DTO而非实体
     */
    @PostMapping
    public ResponseEntity<UserDTO> createUser(@RequestBody CreateUserRequest request) {
        User user = userManagementService.createUser(
                request.getName(), 
                request.getEmail(), 
                request.getPhone()
        );
        UserDTO userDTO = userMapper.toDTO(user);
        return ResponseEntity.ok(userDTO);
    }
    
    /**
     * 获取用户 - 演示从直接返回User实体改为返回DTO的适配过程
     * 
     * 原始代码（有问题的方式）：
     * @GetMapping("/{id}")
     * public ResponseEntity<User> getUser(@PathVariable Long id) {
     *     User user = userManagementService.getUserById(id);
     *     if (user != null) {
     *         return ResponseEntity.ok(user);  // ❌ 直接暴露实体
     *     }
     *     return ResponseEntity.notFound().build();
     * }
     * 
     * 改进后的代码（推荐方式）：
     */
    @GetMapping("/{id}")
    public ResponseEntity<UserDTO> getUser(@PathVariable Long id) {
        User user = userManagementService.getUserById(id);
        if (user != null) {
            // ✅ 通过Mapper转换为DTO，隔离实体变更影响
            UserDTO userDTO = userMapper.toDTO(user);
            return ResponseEntity.ok(userDTO);
        }
        return ResponseEntity.notFound().build();
    }
    
    /**
     * 为了演示对比，这里保留原始的有问题的方法
     * 当User实体变更时，这个方法会直接受到影响
     */
    @GetMapping("/legacy/{id}")
    @Deprecated
    public ResponseEntity<User> getUserLegacy(@PathVariable Long id) {
        User user = userManagementService.getUserById(id);
        if (user != null) {
            // ❌ 问题：直接返回实体，User结构变更会影响API响应
            return ResponseEntity.ok(user);
        }
        return ResponseEntity.notFound().build();
    }
    
    /**
     * 更新用户 - 返回DTO
     */
    @PutMapping("/{id}")
    public ResponseEntity<UserDTO> updateUser(@PathVariable Long id, @RequestBody UpdateUserRequest request) {
        User user = userManagementService.updateUser(
                id, 
                request.getName(), 
                request.getEmail(), 
                request.getPhone()
        );
        if (user != null) {
            UserDTO userDTO = userMapper.toDTO(user);
            return ResponseEntity.ok(userDTO);
        }
        return ResponseEntity.notFound().build();
    }
    
    /**
     * 删除用户
     */
    @DeleteMapping("/{id}")
    public ResponseEntity<Void> deleteUser(@PathVariable Long id) {
        userManagementService.deleteUser(id);
        return ResponseEntity.ok().build();
    }
    
    /**
     * 获取所有用户 - 返回DTO列表
     */
    @GetMapping
    public ResponseEntity<List<UserDTO>> getAllUsers() {
        List<User> users = userManagementService.getAllActiveUsers();
        List<UserDTO> userDTOs = users.stream()
                .map(userMapper::toDTO)
                .collect(Collectors.toList());
        return ResponseEntity.ok(userDTOs);
    }
    
    /**
     * 更新员工信息 - 返回DTO
     */
    @PutMapping("/{id}/employee")
    public ResponseEntity<UserDTO> updateEmployeeInfo(@PathVariable Long id, @RequestBody EmployeeUpdateRequest request) {
        User user = hrService.updateEmployeeInfo(
                id, 
                request.getDepartment(), 
                request.getPosition(), 
                request.getSalary()
        );
        if (user != null) {
            UserDTO userDTO = userMapper.toDTO(user);
            return ResponseEntity.ok(userDTO);
        }
        return ResponseEntity.notFound().build();
    }
    
    /**
     * 获取部门员工 - 返回DTO列表
     */
    @GetMapping("/department/{department}")
    public ResponseEntity<List<UserDTO>> getEmployeesByDepartment(@PathVariable String department) {
        List<User> employees = hrService.getEmployeesByDepartment(department);
        List<UserDTO> employeeDTOs = employees.stream()
                .map(userMapper::toDTO)
                .collect(Collectors.toList());
        return ResponseEntity.ok(employeeDTOs);
    }
    
    /**
     * 锁定用户
     */
    @PostMapping("/{id}/lock")
    public ResponseEntity<Void> lockUser(@PathVariable Long id) {
        securityService.lockUser(id);
        return ResponseEntity.ok().build();
    }
    
    /**
     * 解锁用户
     */
    @PostMapping("/{id}/unlock")
    public ResponseEntity<Void> unlockUser(@PathVariable Long id) {
        securityService.unlockUser(id);
        return ResponseEntity.ok().build();
    }
    
    /**
     * 发送通知
     */
    @PostMapping("/{id}/notify")
    public ResponseEntity<Void> sendNotification(@PathVariable Long id, @RequestBody NotificationRequest request) {
        notificationService.sendEmailNotification(id, request.getMessage());
        return ResponseEntity.ok().build();
    }
    
    /**
     * 获取用户简要信息 - 演示不同的DTO视图
     */
    @GetMapping("/{id}/summary")
    public ResponseEntity<UserSummaryDTO> getUserSummary(@PathVariable Long id) {
        User user = userManagementService.getUserById(id);
        if (user != null) {
            UserSummaryDTO summary = userMapper.toSummaryDTO(user);
            return ResponseEntity.ok(summary);
        }
        return ResponseEntity.notFound().build();
    }
    
    // DTO类 - 与原控制器相同，但可以根据需要调整
    public static class CreateUserRequest {
        private String name;
        private String email;
        private String phone;
        
        // Getters and Setters
        public String getName() { return name; }
        public void setName(String name) { this.name = name; }
        public String getEmail() { return email; }
        public void setEmail(String email) { this.email = email; }
        public String getPhone() { return phone; }
        public void setPhone(String phone) { this.phone = phone; }
    }
    
    public static class UpdateUserRequest {
        private String name;
        private String email;
        private String phone;
        
        // Getters and Setters
        public String getName() { return name; }
        public void setName(String name) { this.name = name; }
        public String getEmail() { return email; }
        public void setEmail(String email) { this.email = email; }
        public String getPhone() { return phone; }
        public void setPhone(String phone) { this.phone = phone; }
    }
    
    public static class EmployeeUpdateRequest {
        private String department;
        private String position;
        private Double salary;
        
        // Getters and Setters
        public String getDepartment() { return department; }
        public void setDepartment(String department) { this.department = department; }
        public String getPosition() { return position; }
        public void setPosition(String position) { this.position = position; }
        public Double getSalary() { return salary; }
        public void setSalary(Double salary) { this.salary = salary; }
    }
    
    public static class NotificationRequest {
        private String message;
        
        // Getters and Setters
        public String getMessage() { return message; }
        public void setMessage(String message) { this.message = message; }
    }
    
    /**
     * 用户简要信息DTO - 演示不同视图
     */
    public static class UserSummaryDTO {
        private Long id;
        private String name;
        private String email;
        private String status;
        
        public UserSummaryDTO(Long id, String name, String email, String status) {
            this.id = id;
            this.name = name;
            this.email = email;
            this.status = status;
        }
        
        // Getters and Setters
        public Long getId() { return id; }
        public void setId(Long id) { this.id = id; }
        public String getName() { return name; }
        public void setName(String name) { this.name = name; }
        public String getEmail() { return email; }
        public void setEmail(String email) { this.email = email; }
        public String getStatus() { return status; }
        public void setStatus(String status) { this.status = status; }
    }
}