package com.example.refactor.dto;

import com.example.refactor.entity.User;
import com.example.refactor.controller.UserControllerV2.UserSummaryDTO;
import org.springframework.stereotype.Component;

/**
 * 用户实体与DTO之间的映射器
 * 关键作用：
 * 1. 封装实体到DTO的转换逻辑
 * 2. 当User实体结构变化时，只需修改此类
 * 3. 支持不同的DTO视图（完整、简要等）
 * 4. 可以添加数据验证和转换逻辑
 */
@Component
public class UserMapper {
    
    /**
     * 将User实体转换为完整的UserDTO
     * 当User实体结构变化时，主要修改此方法
     */
    public UserDTO toDTO(User user) {
        if (user == null) {
            return null;
        }
        
        UserDTO dto = new UserDTO();
        
        // 基本信息映射
        dto.setId(user.getId());
        dto.setName(user.getName());
        dto.setEmail(user.getEmail());
        dto.setPhone(user.getPhone());
        dto.setAddress(user.getAddress());
        dto.setCreatedAt(user.getCreatedAt());
        dto.setUpdatedAt(user.getUpdatedAt());
        
        // 员工信息映射
        dto.setDepartment(user.getDepartment());
        dto.setPosition(user.getPosition());
        dto.setSalary(user.getSalary());
        dto.setHireDate(user.getHireDate());
        
        // 安全信息映射（只暴露必要信息）
        dto.setStatus(user.getStatus());
        dto.setLastLoginTime(user.getLastLoginTime());
        dto.setIsLocked(user.getIsLocked());
        
        return dto;
    }
    
    /**
     * 将UserDTO转换为User实体
     * 用于创建或更新操作
     */
    public User toEntity(UserDTO dto) {
        if (dto == null) {
            return null;
        }
        
        User user = new User();
        
        // 基本信息映射
        user.setId(dto.getId());
        user.setName(dto.getName());
        user.setEmail(dto.getEmail());
        user.setPhone(dto.getPhone());
        user.setAddress(dto.getAddress());
        user.setCreatedAt(dto.getCreatedAt());
        user.setUpdatedAt(dto.getUpdatedAt());
        
        // 员工信息映射
        user.setDepartment(dto.getDepartment());
        user.setPosition(dto.getPosition());
        user.setSalary(dto.getSalary());
        user.setHireDate(dto.getHireDate());
        
        // 安全信息映射
        user.setStatus(dto.getStatus());
        user.setLastLoginTime(dto.getLastLoginTime());
        user.setIsLocked(dto.getIsLocked());
        
        return user;
    }
    
    /**
     * 转换为简要DTO - 只包含关键信息
     * 演示不同的视图模式
     */
    public UserSummaryDTO toSummaryDTO(User user) {
        if (user == null) {
            return null;
        }
        
        return new UserSummaryDTO(
            user.getId(),
            user.getName(),
            user.getEmail(),
            user.getStatus()
        );
    }
    
    /**
     * 更新现有User实体的字段
     * 用于部分更新操作，避免覆盖不应该修改的字段
     */
    public void updateEntityFromDTO(User existingUser, UserDTO dto) {
        if (existingUser == null || dto == null) {
            return;
        }
        
        // 只更新允许修改的字段
        if (dto.getName() != null) {
            existingUser.setName(dto.getName());
        }
        if (dto.getEmail() != null) {
            existingUser.setEmail(dto.getEmail());
        }
        if (dto.getPhone() != null) {
            existingUser.setPhone(dto.getPhone());
        }
        if (dto.getAddress() != null) {
            existingUser.setAddress(dto.getAddress());
        }
        
        // 员工信息更新
        if (dto.getDepartment() != null) {
            existingUser.setDepartment(dto.getDepartment());
        }
        if (dto.getPosition() != null) {
            existingUser.setPosition(dto.getPosition());
        }
        if (dto.getSalary() != null) {
            existingUser.setSalary(dto.getSalary());
        }
        
        // 注意：某些字段如ID、创建时间等不应该被更新
        // 安全相关字段通常通过专门的服务方法更新
    }
    
    /**
     * 创建员工信息DTO
     * 只包含员工相关信息
     */
    public EmployeeDTO toEmployeeDTO(User user) {
        if (user == null) {
            return null;
        }
        
        EmployeeDTO dto = new EmployeeDTO();
        dto.setUserId(user.getId());
        dto.setName(user.getName());
        dto.setEmail(user.getEmail());
        dto.setDepartment(user.getDepartment());
        dto.setPosition(user.getPosition());
        dto.setSalary(user.getSalary());
        dto.setHireDate(user.getHireDate());
        
        return dto;
    }
    
    /**
     * 员工信息DTO
     */
    public static class EmployeeDTO {
        private Long userId;
        private String name;
        private String email;
        private String department;
        private String position;
        private Double salary;
        private java.time.LocalDateTime hireDate;
        
        // Getters and Setters
        public Long getUserId() { return userId; }
        public void setUserId(Long userId) { this.userId = userId; }
        public String getName() { return name; }
        public void setName(String name) { this.name = name; }
        public String getEmail() { return email; }
        public void setEmail(String email) { this.email = email; }
        public String getDepartment() { return department; }
        public void setDepartment(String department) { this.department = department; }
        public String getPosition() { return position; }
        public void setPosition(String position) { this.position = position; }
        public Double getSalary() { return salary; }
        public void setSalary(Double salary) { this.salary = salary; }
        public java.time.LocalDateTime getHireDate() { return hireDate; }
        public void setHireDate(java.time.LocalDateTime hireDate) { this.hireDate = hireDate; }
    }
}