package com.example.refactor.interceptor;

import com.example.refactor.entity.User;
import com.example.refactor.service.SnapshotService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 快照拦截器
 * 统一处理所有用户相关的操作，自动应用快照逻辑
 */
@Component
public class SnapshotInterceptor {
    
    @Autowired(required = false)
    private SnapshotService snapshotService;
    
    /**
     * 拦截单个用户查询操作
     * @param userId 用户ID
     * @param queryFunction 原始查询函数
     * @return 处理后的用户数据
     */
    public User interceptUserQuery(Long userId, Function<Long, User> queryFunction) {
        if (snapshotService == null) {
            return queryFunction.apply(userId);
        }
        
        return snapshotService.getUserWithSnapshot(userId);
    }
    
    /**
     * 拦截用户列表查询操作
     * @param queryFunction 原始查询函数
     * @return 处理后的用户列表
     */
    public List<User> interceptUserListQuery(Function<Void, List<User>> queryFunction) {
        List<User> users = queryFunction.apply(null);
        
        if (snapshotService == null) {
            return users;
        }
        
        return users.stream()
                .map(user -> snapshotService.getUserWithSnapshot(user.getId()))
                .filter(user -> user != null)
                .collect(Collectors.toList());
    }
    
    /**
     * 拦截用户更新操作
     * @param userId 用户ID
     * @param updateFunction 原始更新函数
     * @return 更新后的用户数据
     */
    public User interceptUserUpdate(Long userId, Function<Long, User> updateFunction) {
        // 如果启用了快照功能，在更新前创建快照
        if (snapshotService != null) {
            User currentUser = snapshotService.getUserWithSnapshot(userId);
            if (currentUser != null && currentUser.getSnapshotId() != null) {
                // 用户正在使用快照，创建新快照
                snapshotService.createSnapshot(userId, "用户信息更新");
            }
        }
        
        return updateFunction.apply(userId);
    }
    
    /**
     * 拦截用户删除操作
     * @param userId 用户ID
     * @param deleteFunction 原始删除函数
     */
    public void interceptUserDelete(Long userId, Function<Long, Void> deleteFunction) {
        // 如果启用了快照功能，在删除前创建快照
        if (snapshotService != null) {
            User currentUser = snapshotService.getUserWithSnapshot(userId);
            if (currentUser != null && currentUser.getSnapshotId() != null) {
                // 用户正在使用快照，创建新快照
                snapshotService.createSnapshot(userId, "用户删除前备份");
            }
        }
        
        deleteFunction.apply(userId);
    }
    
    /**
     * 拦截用户创建操作
     * @param createFunction 原始创建函数
     * @return 创建后的用户数据
     */
    public User interceptUserCreate(Function<Void, User> createFunction) {
        // 创建操作不需要快照处理，直接执行
        return createFunction.apply(null);
    }
} 