package com.example.refactor.service;

import com.example.refactor.entity.User;
import com.example.refactor.entity.UserSnapshot;
import com.example.refactor.repository.UserRepository;
import com.example.refactor.repository.UserSnapshotRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Optional;
import java.util.UUID;

/**
 * 快照服务类
 * 负责用户数据的快照创建、管理和查询
 * 通过配置控制是否启用
 */
@Service
@Transactional
@ConditionalOnProperty(name = "app.snapshot.enabled", havingValue = "true", matchIfMissing = false)
public class SnapshotService {
    
    @Autowired
    private UserRepository userRepository;
    
    @Autowired
    private UserSnapshotRepository userSnapshotRepository;
    
    /**
     * 为用户创建快照
     * @param userId 用户ID
     * @param reason 快照创建原因
     * @return 快照ID
     */
    public String createSnapshot(Long userId, String reason) {
        User user = userRepository.findById(userId).orElse(null);
        if (user == null) {
            throw new IllegalArgumentException("用户不存在: " + userId);
        }
        
        // 生成唯一的快照ID
        String snapshotId = generateSnapshotId();
        
        // 创建快照
        UserSnapshot snapshot = UserSnapshot.fromUser(user, snapshotId, reason);
        userSnapshotRepository.save(snapshot);
        
        return snapshotId;
    }
    
    /**
     * 为用户创建快照并设置用户使用该快照
     * @param userId 用户ID
     * @param reason 快照创建原因
     * @return 快照ID
     */
    public String createAndApplySnapshot(Long userId, String reason) {
        String snapshotId = createSnapshot(userId, reason);
        
        // 设置用户使用该快照
        User user = userRepository.findById(userId).orElse(null);
        if (user != null) {
            user.setSnapshotId(snapshotId);
            userRepository.save(user);
        }
        
        return snapshotId;
    }
    
    /**
     * 获取用户数据（优先使用快照）
     * @param userId 用户ID
     * @return 用户数据
     */
    public User getUserWithSnapshot(Long userId) {
        User user = userRepository.findById(userId).orElse(null);
        if (user == null) {
            return null;
        }
        
        // 如果用户设置了快照ID，则使用快照数据
        if (user.getSnapshotId() != null && !user.getSnapshotId().trim().isEmpty()) {
            Optional<UserSnapshot> snapshotOpt = userSnapshotRepository.findBySnapshotId(user.getSnapshotId());
            if (snapshotOpt.isPresent()) {
                return snapshotOpt.get().toUser();
            }
        }
        
        // 否则返回原始用户数据
        return user;
    }
    
    /**
     * 获取用户的所有快照
     * @param userId 用户ID
     * @return 快照列表
     */
    public List<UserSnapshot> getUserSnapshots(Long userId) {
        return userSnapshotRepository.findByUserIdOrderBySnapshotCreateTimeDesc(userId);
    }
    
    /**
     * 根据快照ID获取快照
     * @param snapshotId 快照ID
     * @return 快照数据
     */
    public UserSnapshot getSnapshot(String snapshotId) {
        return userSnapshotRepository.findBySnapshotId(snapshotId).orElse(null);
    }
    
    /**
     * 将用户恢复到指定快照
     * @param userId 用户ID
     * @param snapshotId 快照ID
     * @return 是否成功
     */
    public boolean restoreToSnapshot(Long userId, String snapshotId) {
        Optional<UserSnapshot> snapshotOpt = userSnapshotRepository.findByUserIdAndSnapshotId(userId, snapshotId);
        if (!snapshotOpt.isPresent()) {
            return false;
        }
        
        User user = userRepository.findById(userId).orElse(null);
        if (user == null) {
            return false;
        }
        
        // 设置用户使用该快照
        user.setSnapshotId(snapshotId);
        userRepository.save(user);
        
        return true;
    }
    
    /**
     * 清除用户的快照设置，恢复到原始数据
     * @param userId 用户ID
     * @return 是否成功
     */
    public boolean clearSnapshot(Long userId) {
        User user = userRepository.findById(userId).orElse(null);
        if (user == null) {
            return false;
        }
        
        user.setSnapshotId(null);
        userRepository.save(user);
        
        return true;
    }
    
    /**
     * 删除快照
     * @param snapshotId 快照ID
     * @return 是否成功
     */
    public boolean deleteSnapshot(String snapshotId) {
        Optional<UserSnapshot> snapshotOpt = userSnapshotRepository.findBySnapshotId(snapshotId);
        if (!snapshotOpt.isPresent()) {
            return false;
        }
        
        UserSnapshot snapshot = snapshotOpt.get();
        
        // 检查是否有用户正在使用此快照
        List<User> usersUsingSnapshot = userRepository.findBySnapshotId(snapshotId);
        if (!usersUsingSnapshot.isEmpty()) {
            throw new IllegalStateException("无法删除快照，有用户正在使用此快照");
        }
        
        userSnapshotRepository.delete(snapshot);
        return true;
    }
    
    /**
     * 检查快照是否存在
     * @param snapshotId 快照ID
     * @return 是否存在
     */
    public boolean snapshotExists(String snapshotId) {
        return userSnapshotRepository.existsBySnapshotId(snapshotId);
    }
    
    /**
     * 生成唯一的快照ID
     * @return 快照ID
     */
    private String generateSnapshotId() {
        return "SNAP_" + UUID.randomUUID().toString().replace("-", "").substring(0, 16);
    }
} 