package com.example.refactor.aspect;

import com.example.refactor.entity.User;
import com.example.refactor.service.UserManagementService;
import com.example.refactor.service.HRService;
import com.example.refactor.service.SecurityService;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

import static org.junit.jupiter.api.Assertions.*;

/**
 * 用户实体切面测试类
 * 验证方法签名模式拦截是否正常工作
 */
@SpringBootTest
@ActiveProfiles("test")
@Transactional
public class UserEntityAspectTest {
    
    @Autowired
    private UserManagementService userManagementService;
    
    @Autowired
    private HRService hrService;
    
    @Autowired
    private SecurityService securityService;
    
    private User testUser;
    
    @BeforeEach
    void setUp() {
        // 创建测试用户
        testUser = userManagementService.createUser("测试用户", "test@example.com", "13800138000");
    }
    
    @Test
    void testGetUserByIdWithMethodSignature() {
        // 测试查询方法是否被方法签名模式拦截
        User user = userManagementService.getUserById(testUser.getId());
        
        // 验证返回的是原始数据（因为快照功能未启用）
        assertNotNull(user);
        assertEquals("测试用户", user.getName());
        assertEquals("test@example.com", user.getEmail());
        assertNull(user.getSnapshotId());
    }
    
    @Test
    void testUpdateUserWithMethodSignature() {
        // 测试更新方法是否被方法签名模式拦截
        User updatedUser = userManagementService.updateUser(
                testUser.getId(), 
                "更新后的用户", 
                "updated@example.com", 
                "13800138001"
        );
        
        // 验证更新成功
        assertNotNull(updatedUser);
        assertEquals("更新后的用户", updatedUser.getName());
        assertEquals("updated@example.com", updatedUser.getEmail());
    }
    
    @Test
    void testDeleteUserWithMethodSignature() {
        // 测试删除方法是否被方法签名模式拦截
        userManagementService.deleteUser(testUser.getId());
        
        // 验证删除成功（状态变为DELETED）
        User deletedUser = userManagementService.getUserById(testUser.getId());
        assertNotNull(deletedUser);
        assertEquals("DELETED", deletedUser.getStatus());
    }
    
    @Test
    void testGetAllActiveUsersWithMethodSignature() {
        // 创建多个用户
        userManagementService.createUser("用户1", "user1@example.com", "13800138001");
        userManagementService.createUser("用户2", "user2@example.com", "13800138002");
        
        // 测试列表查询方法是否被方法签名模式拦截
        var users = userManagementService.getAllActiveUsers();
        
        // 验证返回的是原始数据
        assertNotNull(users);
        assertTrue(users.size() >= 3); // 至少包含3个用户
        
        // 验证所有用户都没有快照ID
        users.forEach(user -> assertNull(user.getSnapshotId()));
    }
    
    @Test
    void testHRServiceWithMethodSignature() {
        // 测试HR服务的方法是否被拦截
        User employee = hrService.updateEmployeeInfo(testUser.getId(), "技术部", "工程师", 8000.0);
        assertNotNull(employee);
        assertEquals("技术部", employee.getDepartment());
        assertEquals("工程师", employee.getPosition());
        
        List<User> employees = hrService.getEmployeesByDepartment("技术部");
        assertNotNull(employees);
        assertTrue(employees.size() >= 1);
    }
    
    @Test
    void testSecurityServiceWithMethodSignature() {
        // 测试安全服务的方法是否被拦截
        securityService.lockUser(testUser.getId());
        
        User lockedUser = userManagementService.getUserById(testUser.getId());
        assertNotNull(lockedUser);
        assertEquals("LOCKED", lockedUser.getStatus());
        
        securityService.unlockUser(testUser.getId());
        
        User unlockedUser = userManagementService.getUserById(testUser.getId());
        assertNotNull(unlockedUser);
        assertEquals("ACTIVE", unlockedUser.getStatus());
    }
    
    @Test
    void testPromoteEmployeeWithMethodSignature() {
        // 测试员工晋升方法是否被拦截
        User promotedUser = hrService.promoteEmployee(testUser.getId(), "高级工程师", 12000.0);
        assertNotNull(promotedUser);
        assertEquals("高级工程师", promotedUser.getPosition());
        assertEquals(12000.0, promotedUser.getSalary());
    }
    
    @Test
    void testTransferEmployeeWithMethodSignature() {
        // 测试员工调动方法是否被拦截
        User transferredUser = hrService.transferEmployee(testUser.getId(), "产品部");
        assertNotNull(transferredUser);
        assertEquals("产品部", transferredUser.getDepartment());
    }
} 