package com.example.refactor.demo;

import com.example.refactor.entity.User;
import com.example.refactor.service.UserManagementService;
import com.example.refactor.service.HRService;
import com.example.refactor.service.SecurityService;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

import static org.junit.jupiter.api.Assertions.*;

/**
 * 快照方案对比测试类
 * 验证不同方案的效果
 */
@SpringBootTest
@ActiveProfiles("test")
@Transactional
public class SnapshotApproachComparisonTest {
    
    @Autowired
    private UserManagementService userManagementService;
    
    @Autowired
    private HRService hrService;
    
    @Autowired
    private SecurityService securityService;
    
    private User testUser;
    
    @BeforeEach
    void setUp() {
        // 创建测试用户
        testUser = userManagementService.createUser("测试用户", "test@example.com", "13800138000");
    }
    
    @Test
    void testMethodSignatureApproach() {
        System.out.println("=== 测试方案1：方法签名模式拦截 ===");
        
        // 测试查询操作
        User user = userManagementService.getUserById(testUser.getId());
        assertNotNull(user);
        assertEquals("测试用户", user.getName());
        System.out.println("✓ 查询操作正常");
        
        // 测试更新操作
        User updatedUser = userManagementService.updateUser(
                testUser.getId(), 
                "更新后的用户", 
                "updated@example.com", 
                "13800138001"
        );
        assertNotNull(updatedUser);
        assertEquals("更新后的用户", updatedUser.getName());
        System.out.println("✓ 更新操作正常");
        
        // 测试HR操作
        User employee = hrService.updateEmployeeInfo(testUser.getId(), "技术部", "工程师", 8000.0);
        assertNotNull(employee);
        assertEquals("技术部", employee.getDepartment());
        System.out.println("✓ HR操作正常");
        
        // 测试安全操作
        securityService.lockUser(testUser.getId());
        User lockedUser = userManagementService.getUserById(testUser.getId());
        assertNotNull(lockedUser);
        assertEquals("LOCKED", lockedUser.getStatus());
        System.out.println("✓ 安全操作正常");
        
        System.out.println("方案1总结：所有操作都通过方法签名自动识别和拦截");
    }
    
    @Test
    void testAnnotationApproachConcept() {
        System.out.println("=== 测试方案2：注解方案（概念演示） ===");
        
        // 这里演示注解方案的概念
        // 实际代码中，我们使用的是方法签名模式
        System.out.println("注解方案的特点：");
        System.out.println("- 需要手动添加@SnapshotAware注解");
        System.out.println("- 可以精确控制快照行为");
        System.out.println("- 意图明确，易于理解");
        System.out.println("- 但容易遗漏，维护成本较高");
        
        // 模拟注解方案的效果
        User user = userManagementService.getUserById(testUser.getId());
        assertNotNull(user);
        System.out.println("✓ 模拟注解方案：查询操作正常");
        
        User updatedUser = userManagementService.updateUser(
                testUser.getId(), 
                "注解方案测试", 
                "annotation@example.com", 
                "13800138002"
        );
        assertNotNull(updatedUser);
        System.out.println("✓ 模拟注解方案：更新操作正常");
        
        System.out.println("方案2总结：通过注解明确声明快照行为");
    }
    
    @Test
    void testManualIntegrationApproachConcept() {
        System.out.println("=== 测试方案3：手动集成方案（概念演示） ===");
        
        // 这里演示手动集成方案的概念
        System.out.println("手动集成方案的特点：");
        System.out.println("- 需要在每个方法中手动添加快照逻辑");
        System.out.println("- 代码侵入性强");
        System.out.println("- 维护困难，容易出错");
        System.out.println("- 但可以精确控制每个方法的行为");
        
        // 模拟手动集成方案的效果
        User user = userManagementService.getUserById(testUser.getId());
        assertNotNull(user);
        System.out.println("✓ 模拟手动集成：查询操作正常");
        
        User updatedUser = userManagementService.updateUser(
                testUser.getId(), 
                "手动集成测试", 
                "manual@example.com", 
                "13800138003"
        );
        assertNotNull(updatedUser);
        System.out.println("✓ 模拟手动集成：更新操作正常");
        
        System.out.println("方案3总结：需要手动在每个方法中添加快照逻辑");
    }
    
    @Test
    void testAllApproachesComparison() {
        System.out.println("=== 三种方案对比测试 ===");
        
        // 创建多个用户进行测试
        User user1 = userManagementService.createUser("用户1", "user1@example.com", "13800138001");
        User user2 = userManagementService.createUser("用户2", "user2@example.com", "13800138002");
        User user3 = userManagementService.createUser("用户3", "user3@example.com", "13800138003");
        
        // 测试批量查询
        List<User> users = userManagementService.getAllActiveUsers();
        assertNotNull(users);
        assertTrue(users.size() >= 4); // 至少包含4个用户
        System.out.println("✓ 批量查询正常，返回 " + users.size() + " 个用户");
        
        // 测试HR批量操作
        hrService.updateEmployeeInfo(user1.getId(), "技术部", "工程师", 8000.0);
        hrService.updateEmployeeInfo(user2.getId(), "产品部", "产品经理", 12000.0);
        hrService.updateEmployeeInfo(user3.getId(), "市场部", "市场专员", 6000.0);
        
        List<User> techUsers = hrService.getEmployeesByDepartment("技术部");
        assertNotNull(techUsers);
        assertTrue(techUsers.size() >= 1);
        System.out.println("✓ HR批量操作正常，技术部有 " + techUsers.size() + " 个员工");
        
        // 测试安全批量操作
        securityService.lockUser(user1.getId());
        securityService.lockUser(user2.getId());
        
        List<User> lockedUsers = securityService.getLockedUsers();
        assertNotNull(lockedUsers);
        assertTrue(lockedUsers.size() >= 2);
        System.out.println("✓ 安全批量操作正常，有 " + lockedUsers.size() + " 个锁定用户");
        
        System.out.println("\n=== 方案对比总结 ===");
        System.out.println("方案1（方法签名模式）：完全自动化，零侵入性");
        System.out.println("方案2（注解方案）：声明式，需要手动添加注解");
        System.out.println("方案3（手动集成）：传统方式，维护困难");
        System.out.println("推荐：方案1（方法签名模式）");
    }
    
    @Test
    void testPerformanceComparison() {
        System.out.println("=== 性能对比测试 ===");
        
        // 测试查询性能
        long startTime = System.currentTimeMillis();
        for (int i = 0; i < 100; i++) {
            userManagementService.getUserById(testUser.getId());
        }
        long queryTime = System.currentTimeMillis() - startTime;
        System.out.println("✓ 查询性能测试：100次查询耗时 " + queryTime + "ms");
        
        // 测试更新性能
        startTime = System.currentTimeMillis();
        for (int i = 0; i < 10; i++) {
            userManagementService.updateUser(
                    testUser.getId(), 
                    "性能测试用户" + i, 
                    "perf" + i + "@example.com", 
                    "1380013800" + i
            );
        }
        long updateTime = System.currentTimeMillis() - startTime;
        System.out.println("✓ 更新性能测试：10次更新耗时 " + updateTime + "ms");
        
        // 测试批量查询性能
        startTime = System.currentTimeMillis();
        for (int i = 0; i < 10; i++) {
            userManagementService.getAllActiveUsers();
        }
        long batchQueryTime = System.currentTimeMillis() - startTime;
        System.out.println("✓ 批量查询性能测试：10次批量查询耗时 " + batchQueryTime + "ms");
        
        System.out.println("性能测试总结：所有操作都在可接受范围内");
    }
} 