package com.example.refactor.service;

import com.example.refactor.entity.User;
import com.example.refactor.entity.UserSnapshot;
import com.example.refactor.repository.UserRepository;
import com.example.refactor.repository.UserSnapshotRepository;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

import static org.junit.jupiter.api.Assertions.*;

/**
 * 快照服务测试类
 */
@SpringBootTest
@ActiveProfiles("test")
@Transactional
public class SnapshotServiceTest {
    
    @Autowired
    private SnapshotService snapshotService;
    
    @Autowired
    private UserRepository userRepository;
    
    @Autowired
    private UserSnapshotRepository userSnapshotRepository;
    
    private User testUser;
    
    @BeforeEach
    void setUp() {
        // 创建测试用户
        testUser = new User("测试用户", "test@example.com");
        testUser.setPhone("13800138000");
        testUser.setDepartment("技术部");
        testUser.setPosition("工程师");
        testUser.setSalary(10000.0);
        testUser = userRepository.save(testUser);
    }
    
    @Test
    void testCreateSnapshot() {
        // 创建快照
        String snapshotId = snapshotService.createSnapshot(testUser.getId(), "测试快照");
        
        // 验证快照ID不为空
        assertNotNull(snapshotId);
        assertTrue(snapshotId.startsWith("SNAP_"));
        
        // 验证快照数据被保存
        UserSnapshot snapshot = userSnapshotRepository.findBySnapshotId(snapshotId).orElse(null);
        assertNotNull(snapshot);
        assertEquals(testUser.getId(), snapshot.getUserId());
        assertEquals("测试快照", snapshot.getSnapshotReason());
        assertEquals(testUser.getName(), snapshot.getName());
        assertEquals(testUser.getEmail(), snapshot.getEmail());
        assertEquals(testUser.getDepartment(), snapshot.getDepartment());
    }
    
    @Test
    void testCreateAndApplySnapshot() {
        // 创建并应用快照
        String snapshotId = snapshotService.createAndApplySnapshot(testUser.getId(), "应用快照");
        
        // 验证快照ID不为空
        assertNotNull(snapshotId);
        
        // 验证用户被设置为使用快照
        User updatedUser = userRepository.findById(testUser.getId()).orElse(null);
        assertNotNull(updatedUser);
        assertEquals(snapshotId, updatedUser.getSnapshotId());
    }
    
    @Test
    void testGetUserWithSnapshot() {
        // 创建快照
        String snapshotId = snapshotService.createSnapshot(testUser.getId(), "测试快照");
        
        // 设置用户使用快照
        testUser.setSnapshotId(snapshotId);
        userRepository.save(testUser);
        
        // 获取带快照的用户数据
        User userWithSnapshot = snapshotService.getUserWithSnapshot(testUser.getId());
        
        // 验证返回的是快照数据
        assertNotNull(userWithSnapshot);
        assertEquals(snapshotId, userWithSnapshot.getSnapshotId());
        assertEquals(testUser.getName(), userWithSnapshot.getName());
        assertEquals(testUser.getEmail(), userWithSnapshot.getEmail());
    }
    
    @Test
    void testGetUserWithoutSnapshot() {
        // 获取不带快照的用户数据
        User userWithoutSnapshot = snapshotService.getUserWithSnapshot(testUser.getId());
        
        // 验证返回的是原始数据
        assertNotNull(userWithoutSnapshot);
        assertNull(userWithoutSnapshot.getSnapshotId());
        assertEquals(testUser.getName(), userWithoutSnapshot.getName());
        assertEquals(testUser.getEmail(), userWithoutSnapshot.getEmail());
    }
    
    @Test
    void testGetUserSnapshots() {
        // 创建多个快照
        snapshotService.createSnapshot(testUser.getId(), "快照1");
        snapshotService.createSnapshot(testUser.getId(), "快照2");
        snapshotService.createSnapshot(testUser.getId(), "快照3");
        
        // 获取用户的所有快照
        List<UserSnapshot> snapshots = snapshotService.getUserSnapshots(testUser.getId());
        
        // 验证快照数量
        assertEquals(3, snapshots.size());
        
        // 验证快照按时间倒序排列
        assertTrue(snapshots.get(0).getSnapshotCreateTime().after(snapshots.get(1).getSnapshotCreateTime()));
    }
    
    @Test
    void testRestoreToSnapshot() {
        // 创建快照
        String snapshotId = snapshotService.createSnapshot(testUser.getId(), "测试快照");
        
        // 恢复到快照
        boolean success = snapshotService.restoreToSnapshot(testUser.getId(), snapshotId);
        
        // 验证恢复成功
        assertTrue(success);
        
        // 验证用户被设置为使用快照
        User restoredUser = userRepository.findById(testUser.getId()).orElse(null);
        assertNotNull(restoredUser);
        assertEquals(snapshotId, restoredUser.getSnapshotId());
    }
    
    @Test
    void testClearSnapshot() {
        // 创建并应用快照
        String snapshotId = snapshotService.createAndApplySnapshot(testUser.getId(), "测试快照");
        
        // 清除快照设置
        boolean success = snapshotService.clearSnapshot(testUser.getId());
        
        // 验证清除成功
        assertTrue(success);
        
        // 验证用户不再使用快照
        User clearedUser = userRepository.findById(testUser.getId()).orElse(null);
        assertNotNull(clearedUser);
        assertNull(clearedUser.getSnapshotId());
    }
    
    @Test
    void testDeleteSnapshot() {
        // 创建快照
        String snapshotId = snapshotService.createSnapshot(testUser.getId(), "测试快照");
        
        // 删除快照
        boolean success = snapshotService.deleteSnapshot(snapshotId);
        
        // 验证删除成功
        assertTrue(success);
        
        // 验证快照不存在
        assertFalse(snapshotService.snapshotExists(snapshotId));
    }
    
    @Test
    void testDeleteSnapshotWithUserUsing() {
        // 创建并应用快照
        String snapshotId = snapshotService.createAndApplySnapshot(testUser.getId(), "测试快照");
        
        // 尝试删除快照（应该失败）
        assertThrows(IllegalStateException.class, () -> {
            snapshotService.deleteSnapshot(snapshotId);
        });
    }
    
    @Test
    void testSnapshotExists() {
        // 创建快照
        String snapshotId = snapshotService.createSnapshot(testUser.getId(), "测试快照");
        
        // 验证快照存在
        assertTrue(snapshotService.snapshotExists(snapshotId));
        
        // 验证不存在的快照
        assertFalse(snapshotService.snapshotExists("NON_EXISTENT_SNAPSHOT"));
    }
} 